/*
Copyright (c) 2013, Groupon, Inc.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

Neither the name of GROUPON nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// Generated by CoffeeScript 2.0.0-beta7
void function () {
  var crop, encode, getView, ndarray, parsePng, pick, Pixel, Png, RGBA_CHANNELS, validate;
  RGBA_CHANNELS = 4;
  ndarray = require('ndarray');
  parsePng = require('./parse_png');
  pick = require('underscore').pick;
  Pixel = require('./diff_all/pixel');
  Png = require('png').Png;
  getView = function (image, y1, x1, height, width) {
    var view;
    view = image.lo(y1, x1, 0);
    return view.hi(height, width, 0);
  };
  crop = function (image, y1, x1, height, width) {
    var cropped, i, j, view;
    view = getView(image, y1, x1, height, width);
    cropped = [];
    for (var cache$ = function () {
          var accum$;
          accum$ = [];
          for (var i$ = 0; 0 <= view.shape[0] - 1 ? i$ <= view.shape[0] - 1 : i$ >= view.shape[0] - 1; 0 <= view.shape[0] - 1 ? ++i$ : --i$)
            accum$.push(i$);
          return accum$;
        }.apply(this, arguments), i$ = 0, length$ = cache$.length; i$ < length$; ++i$) {
      i = cache$[i$];
      for (var cache$1 = function () {
            var accum$;
            accum$ = [];
            for (var i$1 = 0; 0 <= view.shape[1] - 1 ? i$1 <= view.shape[1] - 1 : i$1 >= view.shape[1] - 1; 0 <= view.shape[1] - 1 ? ++i$1 : --i$1)
              accum$.push(i$1);
            return accum$;
          }.apply(this, arguments), i$1 = 0, length$1 = cache$1.length; i$1 < length$1; ++i$1) {
        j = cache$1[i$1];
        cropped.push(view.get(i, j, 0));
        cropped.push(view.get(i, j, 1));
        cropped.push(view.get(i, j, 2));
        cropped.push(view.get(i, j, 3));
      }
    }
    return {
      data: cropped,
      height: view.shape[0],
      width: view.shape[1]
    };
  };
  module.exports = function (imageData, section) {
    var image, shape;
    if (typeof imageData === 'string')
      imageData = new Buffer(imageData, 'base64');
    image = parsePng(imageData);
    validate(image, section);
    shape = [
      image.height,
      image.width,
      RGBA_CHANNELS
    ];
    image = ndarray(image.data, shape);
    return encode(crop(image, section.y, section.x, section.height, section.width));
  };
  encode = function (imageData) {
    var data, height, png, width;
    height = imageData.height;
    width = imageData.width;
    data = new Buffer(imageData.data);
    data = Pixel.invertAlpha(data);
    png = new Png(data, width, height, 'rgba');
    return png.encodeSync().toString('base64');
  };
  validate = function (image, section) {
    var imageProperties, oob, sectionProperties;
    sectionProperties = JSON.stringify(section);
    imageProperties = JSON.stringify(pick(image, [
      'height',
      'width'
    ]));
    oob = new Error('Crop section ' + sectionProperties + ' out of bounds for image ' + imageProperties);
    if (section.x < 0)
      throw oob;
    if (section.y < 0)
      throw oob;
    if (section.width < 0)
      throw oob;
    if (section.height < 0)
      throw oob;
    if (image.height < section.y + section.height)
      throw oob;
    if (image.width < section.x + section.width)
      throw oob;
  };
}.call(this);
